import type { Schema } from '$lib/common'
import type { Flow, FlowModule, Job } from '$lib/gen'
import { schemaToObject } from '$lib/utils'
import type { FlowState } from './flowState'

export type PickableProperties = {
	flow_input: Object
	priorIds: Record<string, any>
	previousId: string | undefined
}

type StepPropPicker = {
	pickableProperties: PickableProperties
	extraLib: string
}


type ModuleBranches = FlowModule[][]

function dfs(id: string | undefined, flow: Flow, getParents: boolean = true): FlowModule[] {
	if (id === undefined) {
		return []
	}
	function getSubModules(flowModule: FlowModule): ModuleBranches {
		if (flowModule.value.type === 'forloopflow') {
			return [flowModule.value.modules]
		} else if (flowModule.value.type === 'branchall') {
			return flowModule.value.branches.map((branch) => branch.modules)
		} else if (flowModule.value.type == 'branchone') {
			return [
				...flowModule.value.branches.map((branch) => branch.modules),
				flowModule.value.default
			]
		}
		return []
	}

	function rec(id: string, moduleBranches: ModuleBranches): FlowModule[] | undefined {
		for (let modules of moduleBranches) {


			for (const [i, module] of modules.entries()) {
				if (module.id === id) {
					return getParents ? [module] : modules.slice(0, i + 1).reverse()
				} else {
					const submodules = getSubModules(module)

					if (submodules) {
						let found: FlowModule[] | undefined = rec(id, submodules)

						if (found) {
							return getParents ? [...found, module] : [...found, ...modules.slice(0, i).reverse()]
						}
					}
				}
			}
		}
		return undefined
	}

	return rec(id, [flow.value.modules]) ?? []
}

function getFlowInput(
	parentModules: FlowModule[],
	flowState: FlowState,
	args: any,
	schema: Schema
) {
	const parentModule = parentModules.shift()

	const parentState = parentModule ? flowState[parentModule.id] : undefined

	if (parentState && parentModule) {
		if (parentState.previewArgs) {
			return parentState.previewArgs
		} else {
			const parentFlowInput = getFlowInput(parentModules, flowState, args, schema)

			if (parentModule.value.type === 'forloopflow') {
				return {
					iter: {
						value: "Iteration's value",
						index: "Iteration's index"
					},
					...parentFlowInput,
				}
			} else {
				return parentFlowInput
			}
		}
	} else {
		return schemaToObject(schema, args)
	}
}

export function getStepPropPicker(
	flowState: FlowState,
	parentModule: FlowModule | undefined,
	previousModule: FlowModule | undefined,
	id: string,
	flow: Flow,
	args: any,
	include_node: boolean,
	approvers: boolean = false
): StepPropPicker {
	const flowInput = getFlowInput(dfs(parentModule?.id, flow), flowState, args, flow.schema)

	const previousIds = dfs(id, flow, false).map((x) => x.id)
	if (!include_node) {
		previousIds.shift()
	}


	let priorIds = Object.fromEntries(previousIds.map((id) => [id, flowState[id]?.previewResult ?? {}]).reverse())


	const pickableProperties = {
		flow_input: flowInput,
		priorIds: priorIds,
		previousId: previousIds[0]
	}

	if (approvers && ((previousModule?.suspend?.required_events ?? 0) > 0)) {
		pickableProperties["approvers"] = "The list of approvers"
	}

	return {
		extraLib: buildExtraLib(flowInput, priorIds),
		pickableProperties
	}
}

export function buildExtraLib(flowInput: Record<string, any>, results: Record<string, any>): string {
	return `
/**
* get variable (including secret) at path
* @param {string} path - path of the variable (e.g: g/all/pretty_secret)
*/
export function variable(path: string): string;

/**
* get resource at path
* @param {string} path - path of the resource (e.g: g/all/my_resource)
*/
export function resource(path: string): any;

/**
* flow input as an object
*/
export const flow_input = ${JSON.stringify(flowInput)};

/**
* static params of this same step
*/
export const params: any;

/**
 * result by id
 */
export const results = ${JSON.stringify(results)};
`

}
